data {
  int T;            # number of years
  int J;            # number of states
  int K;            # number of regions
  matrix[J,T-1] y;  # Republican vote share by state, 1976 through 2012
  int region[J];    # Region indexes
  int T_new;        # number of future years
  int T_all;        # T + T_new
  real<lower=0> scale_GP_region_long;
  real<lower=0> scale_GP_state_long;
  real<lower=0> scale_GP_region_short;
  real<lower=0> scale_GP_state_short;
  /*  int home_state_dem[T];
  int home_state_rep[T];
  real beta_home_state; */
  matrix[J,T] two_party_turnout;
  vector[J] forecast_2016;
  matrix[J,J] cov_2016;
}
transformed data {
  real times[T_all];
  matrix[J,T] relative_turnout;
  for (t in 1:T_all){
    times[t] = t;
  }
  for (t in 1:T){
    relative_turnout[,t] = two_party_turnout[,t]/exp(mean(log(two_party_turnout[,t])));
  }
}
parameters {
  matrix[K,T] GP_region;
  matrix[J,T] GP_state;
  vector[T] delta;
  vector[J] gamma;
  vector[K] mu;
  real mu_delta;
  real<lower=0> sigma_delta;
  real<lower=0> sigma_mu;
  vector<lower=0>[K] sigma_gamma;
  real<lower=0> sigma_gamma_0;
  real<lower=0> sigma_error_region;
  real<lower=0> sigma_error_state_1;
  real<lower=0> sigma_error_state_2;
  real<lower=0> sigma_GP_region_long;
  real<lower=0> sigma_GP_state_long;
  real<lower=0> sigma_GP_region_short;
  real<lower=0> sigma_GP_state_short;
#  real<lower=0,upper=1> alpha;
  vector[J] rvote_2016;
}
transformed parameters {
  matrix[T_all,T_all] cov_GP_region;
  matrix[T_all,T_all] cov_GP_state;
  matrix[J,T] turnout_weight;
  cov_GP_region = cov_exp_quad(times, sigma_GP_region_long, scale_GP_region_long) + cov_exp_quad(times, sigma_GP_region_short, scale_GP_region_short);
  cov_GP_state = cov_exp_quad(times, sigma_GP_state_long, scale_GP_state_long) + cov_exp_quad(times, sigma_GP_state_short, scale_GP_state_short);
  for (t in 1:T_all){
    cov_GP_region[t,t] = cov_GP_region[t,t] + sigma_error_region^2;
    cov_GP_state[t,t] = cov_GP_state[t,t] + sigma_error_state_1^2;
  }
  for (t in 1:T){
    for (j in 1:J){
      turnout_weight[j,t] = relative_turnout[j,t]^(-.5);
    }
  }
}
model {
  matrix[J,T] y_pred;
  matrix[T_all,T_all] L_GP_region;
  matrix[T_all,T_all] L_GP_state;
  matrix[J,T] y_full;
  y_full[,1:(T-1)] = y;
  y_full[,T] = rvote_2016;
  rvote_2016 ~ multi_normal(forecast_2016, cov_2016);
  L_GP_region = cholesky_decompose(cov_GP_region);
  L_GP_state = cholesky_decompose(cov_GP_state);
  for (k in 1:K) {
    GP_region[k,] ~ multi_normal_cholesky(rep_vector(0, T), L_GP_region[1:T,1:T]);}
  for (j in 1:J) {
    GP_state[j,] ~ multi_normal_cholesky(rep_vector(0, T), L_GP_state[1:T,1:T]);}
  delta ~ normal(mu_delta, sigma_delta);
  gamma ~ normal(mu[region], sigma_gamma[region]);
  for (t in 1:T){
    for (j in 1:J){
      y_pred[j,t] = delta[t] + gamma[j] + GP_region[region[j],t] + GP_state[j,t];
    }
  }
  to_vector(y_full) ~ normal(to_vector(y_pred), sigma_error_state_2*to_vector(turnout_weight));
  mu ~ normal(0, sigma_mu);
  mu_delta ~ normal(.5, .5);
  sigma_mu ~ normal(0, 1);
  sigma_delta ~ normal(0, 1);
  sigma_gamma ~ lognormal(log(sigma_gamma_0), log(2));
  sigma_gamma_0 ~ normal(0, 1);
  sigma_GP_region_long ~ normal(0, 1);
  sigma_GP_state_long ~ normal(0, 1);
  sigma_GP_region_short ~ normal(0, 1);
  sigma_GP_state_short ~ normal(0, 1);
  sigma_error_region ~ normal(0, 1);
  sigma_error_state_1 ~ normal(0, 1);
  sigma_error_state_2 ~ normal(0, 1);
}
generated quantities {
  matrix[J,T_new] y_new;
  matrix[J,T_new] y_new_pred;
  vector[T_new] delta_new;
  matrix[K,T_new] GP_region_new;
  matrix[J,T_new] GP_state_new;
  for (t in 1:T_new){
    delta_new[t] = normal_rng(.5, sigma_delta);  // Center at 50%, not at mu_delta
  }
  # GP code:
  {
    matrix[K,T_new] mu_region_new;
    matrix[J,T_new] mu_state_new;
    matrix[T_new,T] K_transpose_div_Sigma_region;
    matrix[T_new,T] K_transpose_div_Sigma_state;
    matrix[T_new,T_new] Tau_region_new;
    matrix[T_new,T_new] Tau_state_new;
    matrix[T_new,T_new] L_GP_region_new;
    matrix[T_new,T_new] L_GP_state_new;
    K_transpose_div_Sigma_region = cov_GP_region[(T+1):T_all, 1:T] / cov_GP_region[1:T, 1:T];
    K_transpose_div_Sigma_state = cov_GP_state[(T+1):T_all, 1:T] / cov_GP_state[1:T, 1:T];
    mu_region_new = GP_region * K_transpose_div_Sigma_region';
    mu_state_new = GP_state * K_transpose_div_Sigma_state';
    Tau_region_new = cov_GP_region[(T+1):T_all, (T+1):T_all] - K_transpose_div_Sigma_region * cov_GP_region[1:T, (T+1):T_all];
    Tau_state_new = cov_GP_state[(T+1):T_all, (T+1):T_all] - K_transpose_div_Sigma_state * cov_GP_state[1:T, (T+1):T_all];
    L_GP_region_new = cholesky_decompose(Tau_region_new);
    L_GP_state_new = cholesky_decompose(Tau_state_new);
    for (k in 1:K){
      vector[T_new] sim;
      sim = multi_normal_cholesky_rng(mu_region_new[k,]', L_GP_region_new);
      GP_region_new[k,] = sim';
    }
    for (j in 1:J){
      vector[T_new] sim;
      sim = multi_normal_cholesky_rng(mu_state_new[j,]', L_GP_state_new);
      GP_state_new[j,] = sim';
    }
  }
  for (j in 1:J){
    for (t in 1:T_new){
      y_new_pred[j,t] = delta_new[t] + gamma[j] + GP_region_new[region[j],t] + GP_state_new[j,t];
      y_new[j,t] = normal_rng(y_new_pred[j,t], sigma_error_state_2*turnout_weight[j,T]);
    }
  }
}
