setwd("~/AndrewFiles/research/2012")

library("arm")
library("rstan")
rstan_options(auto_write = TRUE)
options(mc.cores = parallel::detectCores())

past_pres <- read.table("state-election-results-1976-thru-2012.txt", header=TRUE)
forecast_2016_data <- read.table("forecast_2016.txt", header=TRUE)
cov_2016_data <- read.table("cov_2016.txt", header=TRUE)

T <- 11
T_new <- 3
J <- 50
K <- 10

state_groups <- list(c("ME","NH","VT","MA","RI","CT"),
                      c("NY","NJ","PA","MD","DE"),
                      c("OH","MI","IL","WI","MN"),
                      c("WA","OR","CA","HI"),
                      c("AZ","CO","NM","NV"),
                      c("IA","NE","KS","ND","SD"),
                      c("KY","TN","MO","WV","IN"),
                      c("VA","OK","FL","TX","NC"),
                      c("AL","MS","LA","GA","SC","AR"),
                      c("MT","ID","WY","UT","AK"))
region_names <- c("New England", "Mid-Atlantic", "Midwest", "West Coast",  "Southwest","Plains", "Border South", "Outer South", "Deep South", "Mountain  West")
region <- rep(NA, J)
for (k in 1:length(state_groups)){
  ok <- state.abb %in% state_groups[[k]]
  region[(1:J)[ok]] <- k
}

home_state <- read.table("home_state.txt", header=TRUE)

year <- 1972 + 4*(1:(T+T_new))

two_party_turnout <- array(NA, c(J,T))
y <- array(NA, c(J,T-1))
national_mean <- rep(NA, T)
home_state_dem <- rep(NA, T)
home_state_rep <- rep(NA, T)
forecast_2016 <- rep(NA, J)
cov_2016 <- array(NA, c(J,J))
for (j in 1:J){
  for (t in 1:(T-1)){
    state_ok <- as.character(past_pres$state) == state.abb[j]
    year_ok <- past_pres$year == year[t]
    ok <- state_ok & year_ok
    if (sum(ok)!=1) stop(cat(c(j,t)))
    two_party_turnout[j,t] <- past_pres$dem[ok] + past_pres$rep[ok]
    y[j,t] <- past_pres$rep[ok]/two_party_turnout[j,t]
  }
  two_party_turnout[j,T] <- two_party_turnout[j,T-1]
  ok <- as.character(forecast_2016_data$state) == state.abb[j]
  if (sum(ok)!=1) stop(cat(c(j,t)))
  forecast_2016[j] <- forecast_2016_data$rvote[ok]
}
for (j1 in 1:J){
  ok1 <- as.character(cov_2016_data$state) == state.abb[j1]
  for (j2 in 1:J){
    ok2 <- colnames(cov_2016_data) == state.abb[j2]
    cov_2016[j1,j2] <- cov_2016_data[ok1,ok2]
  }
}

for (t in 1:T){
  home_state_dem[t] <- (1:J)[state.abb==as.character(home_state$D_state)[t]]
  home_state_rep[t] <- (1:J)[state.abb==as.character(home_state$R_state)[t]]
}


y_relative <- array(NA, dim(y))
relative_turnout <- array(NA, c(J,T-1))
for (t in 1:(T-1)){
  national_mean[t] <- sum(y[,t]*two_party_turnout[,t]) / sum(two_party_turnout[,t])
  y_relative[,t] <- y[,t] - national_mean[t]
  relative_turnout[,t] <- two_party_turnout[,t]/exp(mean(log(two_party_turnout[,t])));
}

state_ok <- as.character(past_pres$state) == "DC"
year_ok <- past_pres$year == 2012
ok <- state_ok & year_ok
dc_turnout_2012 <- past_pres$dem[ok] + past_pres$rep[ok]

## Fit the Stan model

T_all <- T + T_new
scale_GP_region_long <- 8
scale_GP_state_long <- 8
scale_GP_region_short <- 3
scale_GP_state_short <- 3


fit <- stan("state_forecasts.stan", init=function(){list(rvote_2016=forecast_2016)})
print(fit, pars=c("rvote_2016", "sigma_GP_region_long", "sigma_GP_state_long",  "sigma_GP_region_short",  "sigma_GP_state_short", "sigma_error_region", "sigma_error_state_1", "sigma_error_state_2", "mu_delta", "sigma_delta", "sigma_gamma", "sigma_gamma_0", "mu", "sigma_mu"))

## beta_home_state_hat <- mean(extract(fit)$beta_home_state)
beta_home_state_hat <- 0
home_state_adj <- array(0, c(J, T-1))
for (t in 1:(T-1)){
  home_state_adj[home_state_dem[t], t] <- home_state_adj[home_state_dem[t], t] - beta_home_state_hat
  home_state_adj[home_state_rep[t], t] <- home_state_adj[home_state_rep[t], t] + beta_home_state_hat
}

y_relative_adj <- y_relative - home_state_adj

## Compute quantiles of forecasts

rvote_2016_sims <- extract(fit)$rvote_2016
y_new_sims <- extract(fit)$y_new
n_sims <- dim(y_new_sims)[1]
sims <- array(NA, c(n_sims, J, T_new+1))
sims[,,1] <- rvote_2016_sims
sims[,,2:(T_new+1)] <- y_new_sims
relative_sims <- array(NA, dim(sims))

national_mean_future <- array(NA, c(n_sims, T_new+1))
for (s in 1:n_sims){
  for (t in 1:(T_new+1)){
    national_mean_future[s,t] <- (sum(sims[s,,t]*two_party_turnout[,T]) + .1*dc_turnout_2012) / (sum(two_party_turnout[,T]) + .1*dc_turnout_2012)
    relative_sims[s,,t] <- sims[s,,t] - national_mean_future[s,t]
  }
}

quants <- c(.1, .25, .5, .75, .9)
quants_sims <- array(NA, c(J, T_new+1, length(quants)))
quants_relative_sims <- array(NA, c(J, T_new+1, length(quants)))
for (j in 1:J){
  for (t in 1:(T_new+1)){
    quants_sims[j,t,] <- quantile(sims[,j,t], quants)
    quants_relative_sims[j,t,] <- quantile(relative_sims[,j,t], quants)
  }
}

## Put data together

past_data <- y
dimnames(past_data) <- list(state.name, seq(1976,2012,4))
past_data_relative <- y_relative
dimnames(past_data_relative) <- list(state.name, seq(1976,2012,4))

future_bounds <- array(NA, c(J, 12), dimnames=list(state.name, paste(rep(seq(2016,2028,4), c(rep(3,4))), c("10th","50th","90th"), sep="_")))
future_bounds_relative <- array(NA, c(J, 12), dimnames=list(state.name, paste(rep(seq(2016,2028,4), c(rep(3,4))), c("10th","50th","90th"), sep="_")))
for (t in 1:(T_new+1)){
  future_bounds[,3*(t-1) + (1:3)] <- quants_sims[,t,c(1,3,5)]
  future_bounds_relative[,3*(t-1) + (1:3)] <- quants_relative_sims[,t,c(1,3,5)]
}

write.table(round(past_data, 3), "past_data.txt")
write.table(round(past_data_relative, 3), "past_data_relative_to_national_vote.txt")
write.table(round(future_bounds, 3), "future_bounds.txt")
write.table(round(future_bounds_relative, 3), "future_bounds_relative_to_national_vote.txt")


## Graph data and predictions

pdf("state_forecasts_1.pdf", height=7, width=6)
par(mfrow=c(9,6), oma=c(0,0,3,0), mar=c(1.5,1.5,0,0), mgp=c(.5,.25,0), tck=-.03)
ordering <- order(y[,T-1])
for (jj in 1:50){
  j <- rev(ordering)[jj]
  plot(0, 0, xlim=c(1,T_all), ylim=c(.2, .8), bty="l", xlab="", xaxt="n", ylab="", yaxt="n", type="n")
  polygon(rep(c(T-1, T_all), c(2,2)), c(-1,1,1,-1)*.32 + .5, col="gray90", border=NA)
  polygon(c(rev((T-1):T_all), (T-1):T_all), c(rev(quants_sims[j,,1]), rep(y[j,T-1], 2), quants_sims[j,,5]), col="pink", border=NA)
  lines(1:(T-1), y[j,1:(T-1)], lwd=1)
  for (t in T:T_all){
    lines(rep(t,2), quants_sims[j,t-T+1,c(1,5)], lwd=.5, col="red")
  }
  abline(.5, 0, col="gray", lty=2)
  years <- seq(1980, 2020, 20)
  axis(1, (years-1972)/4, rep("", length(years)))
  if (jj>44){
    axis(1, (years-1972)/4, years, cex.axis=.7)
  }
  axis (2, c(.25, .50, .75), rep("", 3))
  if (jj%%6 == 1){
    axis(2, c(.25,.50,.75), c(".25",".50",".75"), cex.axis=.7)
  }
  mtext(state.name[j], side=3, line=-.2, cex=.6)
}
mtext("Republican vote share in state presidential elections from 1976 and forecast to 2028 (80% prediction intervals)", side=3, line=1.7, cex=.7, outer=TRUE)
dev.off()


pdf("state_forecasts_2.pdf", height=7, width=6)
par(mfrow=c(9,6), oma=c(0,0,3,0), mar=c(1.5,1.5,0,0), mgp=c(.5,.25,0), tck=-.03)
ordering <- order(y[,T-1])
for (jj in 1:50){
  j <- rev(ordering)[jj]
  plot(0, 0, xlim=c(1,T_all), ylim=c(-.3, .3), bty="l", xlab="", xaxt="n", ylab="", yaxt="n", type="n")
  polygon(rep(c(T-1, T_all), c(2,2)), c(-1,1,1,-1)*.32, col="gray90", border=NA)
  polygon(c(rev((T-1):T_all), (T-1):T_all), c(rev(quants_relative_sims[j,,1]), rep(y_relative[j,T-1], 2), quants_relative_sims[j,,5]), col="pink", border=NA)
  lines(1:(T-1), y_relative[j,1:(T-1)], lwd=1)
  for (t in T:T_all){
    lines(rep(t,2), quants_relative_sims[j,t-T+1,c(1,5)], lwd=.5, col="red")
  }
  abline(0, 0, col="gray", lty=2)
  years <- seq(1980, 2020, 20)
  axis(1, (years-1972)/4, rep("", length(years)))
  if (jj>44){
    axis(1, (years-1972)/4, years, cex.axis=.7)
  }
  axis (2, c(-.2, 0, .2), rep("", 3))
  if (jj%%6 == 1){
    axis(2, c(-.2,.2), cex.axis=.7)
  }
  mtext(state.name[j], side=3, line=-.2, cex=.6)
}
mtext("Republican vote share relative to national average from 1976 and forecast to 2028 (80% prediction intervals)", side=3, line=1.7, cex=.7, outer=TRUE)
dev.off()

## For each state, give prob dist for #R in next 3 elections

predict_r_win <- array(NA, c(J, T_new + 1), dimnames=list(state.name, 0:T_new))
y_new_r_win <- ifelse(extract(fit)$y_new > .5, 1, 0)
for (j in 1:J){
  r_win_total <- array(NA, c(n_sims,J))
  for (s in 1:n_sims){
    r_win_total[s,j] <- sum(y_new_r_win[s,j,])
  }
  for (i in 1:(T_new+1)){
    predict_r_win[j,i] <- mean(r_win_total[,j]==(i-1))
  }
}

pfround(predict_r_win[rev(ordering),], 2)
write.table(round(predict_r_win, 2), "predict_r_win.txt")

GP_state_sim <- extract(fit)$GP_state
GP_state_new_sim <- extract(fit)$GP_state_new

pdf("state_forecasts_3.pdf", height=7, width=6)
par(mfrow=c(9,6), oma=c(0,0,3,0), mar=c(1.5,1.5,0,0), mgp=c(.5,.25,0), tck=-.03)
ordering <- order(y[,T-1])
for (jj in 1:50){
  j <- rev(ordering)[jj]
  plot(0, 0, xlim=c(1,T_all), ylim=c(-.15, .15), bty="l", xlab="", xaxt="n", ylab="", yaxt="n", type="n")
  polygon(rep(c(T-1, T_all), c(2,2)), c(-1,1,1,-1)*.32, col="gray90", border=NA)
  for (s in 1:50){
   lines(1:(T-1), GP_state_sim[s,j,1:(T-1)], lwd=.1)
   lines((T-1):T_all, c(GP_state_sim[s,j,(T-1):T], GP_state_new_sim[s,j,1:T_new]), lwd=.1, col="red")
  }
  abline(0, 0, col="gray", lty=2)
  years <- seq(1980, 2020, 20)
  axis(1, (years-1972)/4, rep("", length(years)))
  if (jj>44){
    axis(1, (years-1972)/4, years, cex.axis=.7)
  }
  axis (2, c(-.1, 0, 1), rep("", 3))
  if (jj%%6 == 1){
    axis(2, c(-.1,.1), cex.axis=.7)
  }
  mtext(state.name[j], side=3, line=-.2, cex=.6)
}
mtext("Estimated smooth curves for each state (50 posterior draws)", side=3, line=1.7, cex=.7, outer=TRUE)
dev.off()

